from typing import List
from .marshmallow import JSONSchema, Schema
from .operators import definitions
from ..registry import OperationRegistry, Operation


class Dumper:
    def __init__(self, registry: OperationRegistry, draft=7):
        self._draft = f"http://json-schema.org/draft-0{draft}/schema#"
        self._registry = registry
        self._definitions = {}

    def operation(self, name):
        operation = self._registry.get(name)
        if operation:
            schema = self._wrap_operation(operation)
            return self._wrap(schema)

    def registry(self):
        properties = {}
        for operation in self._registry.all():
            request_schema = operation.request_schema
            response_schema = operation.response_schema
            if callable(request_schema):
                request_schema = request_schema()
            properties[operation.name] = {
                'request': self.dump_schema(request_schema)
            }
            if response_schema:
                if callable(response_schema):
                    response_schema = response_schema()
                properties[operation.name]['response'] = self.dump_schema(response_schema)

        return self._wrap(properties)

    def _wrap_operation(self, operation: Operation):
        request_schema = operation.request_schema
        response_schema = operation.response_schema
        if callable(request_schema):
            request_schema = request_schema()
        operation = {
            'request': self.dump_schema(request_schema)
        }
        if response_schema:
            if callable(response_schema):
                response_schema = response_schema()
            operation['response'] = self.dump_schema(response_schema)

        return operation

    def dump_schema(self, schema: Schema, wrap=False):
        data = JSONSchema(nested=False).dump(schema)
        self._definitions = {**self._definitions, **data.get("definitions", {})}
        dump = self._definitions.pop(schema.__class__.__name__)
        if wrap:
            return self._wrap(dump)
        return dump

    def _wrap(self, properties: dict):
        return {
            "$schema": self._draft,
            "definitions": {**self._definitions, **definitions()},
            "type": "object",
            "properties": properties
        }