from functools import partial
from aiohttp.web_app import Application
from .view import ApiView, SelfDescribeView, HealthCheckView
from .registry import OperationRegistry, REGISTRY_SETTINGS_KEY
from prometheus_async import aio

operations_registry = OperationRegistry()


async def setup_operations_registry(app: Application, registry: OperationRegistry):
    app[REGISTRY_SETTINGS_KEY] = registry


async def setup_health_check_function(app: Application, health_check_func: callable):
    app['health_check_function'] = health_check_func


def setup_server(server: Application, health_check_function: callable = None) -> Application:
    # Define main api route for jsonrpc requests
    server.router.add_post('/api/jsonrpc', ApiView)

    # Define route for self describe
    server.router.add_post('/_describe', SelfDescribeView)

    # Define prometheus metrics route
    server.router.add_get("/metrics", aio.web.server_stats)

    # Register operations registry
    server.on_startup.append(
        partial(
            setup_operations_registry,
            registry=operations_registry
        )
    )
    # Setup healh check function if exists and define proper route
    if health_check_function:
        server.on_startup.append(
            partial(
                setup_health_check_function,
                health_check_func=health_check_function
            )
        )
        server.router.add_get('/health', HealthView)

    return server
