import fastjsonschema
from fastjsonschema.exceptions import JsonSchemaException
from marshmallow import Schema
from .exceptions import JsonRpcException, SCHEMA_NOT_DEFINED_ERROR, SCHEMA_VALIDATION_ERROR
from .schema.dump import Dumper


class SchemaValidator:
    def __init__(self, registry):
        self._operations_registry = registry

    def validate(self, schema: Schema, data):
        if callable(schema):
            schema = schema()
        try:
            self._validator_compile(schema)(data)
        except AttributeError as e:
            raise JsonRpcException(code=SCHEMA_NOT_DEFINED_ERROR, message=str(e))
        except JsonSchemaException as e:
            raise JsonRpcException(code=SCHEMA_VALIDATION_ERROR, context=e.args, message=e.message)

    def _validator_compile(self, schema: Schema):
        dumper = Dumper(self._operations_registry)
        schema = dumper._wrap(dumper.dump_schema(schema).get("properties"))
        return fastjsonschema.compile(schema)
